@@ -1,203 +1,34 @@
-import re
 import itertools
-import operator
-import decimal
 import collections
 import functools
 
 import six
 
 from django.core.exceptions import ValidationError
-from django.core.validators import (
-    MinLengthValidator,
-    MaxLengthValidator,
-)
 
 from flex.exceptions import SafeNestedValidationError
 from flex.constants import (
-    NUMBER,
-    STRING,
-    ARRAY,
     OBJECT,
     EMPTY,
 )
 from flex.decorators import skip_if_not_of_type
-from flex.utils import (
-    prettify_errors,
-    is_value_of_any_type,
-    is_non_string_iterable,
+from flex.context_managers import ErrorCollection
+from flex.validation.common import (
+    skip_if_empty,
+    generate_type_validator,
+    generate_format_validator,
+    generate_required_validator,
+    generate_multiple_of_validator,
+    generate_minimum_validator,
+    generate_maximum_validator,
+    generate_min_length_validator,
+    generate_max_length_validator,
+    generate_min_items_validator,
+    generate_max_items_validator,
+    generate_unique_items_validator,
+    generate_pattern_validator,
+    generate_enum_validator,
 )
-from flex.formats import registry
-
-
-def skip_if_empty(func):
-    @functools.wraps(func)
-    def inner(value, *args, **kwargs):
-        if value is EMPTY:
-            return
-        else:
-            return func(value, *args, **kwargs)
-    return inner
-
-
-@skip_if_empty
-def validate_type(value, types):
-    if not is_value_of_any_type(value, types):
-        raise ValidationError("Invalid Type: {0}".format(value))
-
-
-def generate_type_validator(**kwargs):
-    types = kwargs['type']
-    if not is_non_string_iterable(types):
-        types = (types,)
-    return functools.partial(validate_type, types=types)
-
-
-@skip_if_empty
-@skip_if_not_of_type(NUMBER)
-def validate_multiple_of(value, divisor):
-    """
-    Given a value and a divisor, validate that the value is divisible by the
-    divisor.
-    """
-    if not decimal.Decimal(str(value)) % decimal.Decimal(str(divisor)) == 0:
-        raise ValidationError(
-            "{0} is not a multiple of {1}".format(value, divisor),
-        )
-
-
-def generate_multiple_of_validator(multipleOf, **kwargs):
-    return functools.partial(validate_multiple_of, divisor=multipleOf)
-
-
-@skip_if_empty
-@skip_if_not_of_type(NUMBER)
-def validate_minimum(value, minimum, is_exclusive):
-    if is_exclusive:
-        comparison_text = "greater than"
-        compare_fn = operator.gt
-    else:
-        comparison_text = "greater than or equal to"
-        compare_fn = operator.ge
-
-    if not compare_fn(value, minimum):
-        raise ValidationError(
-            "{0} must be {1} than {2}".format(value, comparison_text, minimum),
-        )
-
-
-def generate_minimum_validator(minimum, exclusiveMinimum=False, **kwargs):
-    return functools.partial(validate_minimum, minimum=minimum, is_exclusive=exclusiveMinimum)
-
-
-@skip_if_empty
-@skip_if_not_of_type(NUMBER)
-def validate_maximum(value, maximum, is_exclusive):
-    if is_exclusive:
-        comparison_text = "less than"
-        compare_fn = operator.lt
-    else:
-        comparison_text = "less than or equal to"
-        compare_fn = operator.le
-
-    if not compare_fn(value, maximum):
-        raise ValidationError(
-            "{0} must be {1} than {2}".format(value, comparison_text, maximum),
-        )
-
-
-def generate_maximum_validator(maximum, exclusiveMaximum=False, **kwargs):
-    return functools.partial(validate_maximum, maximum=maximum, is_exclusive=exclusiveMaximum)
-
-
-def generate_min_length_validator(minLength, **kwargs):
-    return skip_if_empty(skip_if_not_of_type(STRING)(MinLengthValidator(minLength).__call__))
-
-
-def generate_max_length_validator(maxLength, **kwargs):
-    return skip_if_empty(skip_if_not_of_type(STRING)(MaxLengthValidator(maxLength).__call__))
-
-
-@skip_if_empty
-@skip_if_not_of_type(ARRAY)
-def validate_min_items(value, minimum):
-    if len(value) < minimum:
-        raise ValidationError(
-            "Array must have at least {0} items.  It had {1}".format(
-                minimum, len(value),
-            ),
-        )
-
-
-def generate_min_items_validator(minItems, **kwargs):
-    return functools.partial(validate_min_items, minimum=minItems)
-
-
-@skip_if_empty
-@skip_if_not_of_type(ARRAY)
-def validate_max_items(value, maximum):
-    if len(value) > maximum:
-        raise ValidationError(
-            "Array must have no more than {0} items.  It had {1}".format(
-                maximum, len(value),
-            ),
-        )
-
-
-def generate_max_items_validator(maxItems, **kwargs):
-    return functools.partial(validate_max_items, maximum=maxItems)
-
-
-@skip_if_empty
-@skip_if_not_of_type(ARRAY)
-def validate_unique_items(value):
-    counter = collections.Counter(value)
-    dupes = [v for v, count in counter.items() if count > 1]
-    if dupes:
-        raise ValidationError(
-            "Items must be unique.  The following items appeard more than once: {0}".format(
-                repr(dupes),
-            ),
-        )
-
-
-def noop(*args, **kwargs):
-    """
-    No-Op validator that does nothing.
-    """
-    pass
-
-
-def generate_unique_items_generator(uniqueItems, **kwargs):
-    if uniqueItems:
-        return validate_unique_items
-    else:
-        return noop
-
-
-def deep_equal(a, b):
-    """
-    Because of things in python like:
-        >>> 1 == 1.0
-        True
-        >>> 1 == True
-        True
-    """
-    return a == b and isinstance(a, type(b))
-
-
-@skip_if_empty
-def validate_enum(value, options):
-    if not any(deep_equal(value, option) for option in options):
-        raise ValidationError(
-            "Invalid value.  {0} is not one of the available options ({1})".format(
-                value, options,
-            )
-        )
-
-
-def generate_enum_validator(enum, **kwargs):
-    return functools.partial(validate_enum, options=enum)
 
 
 @skip_if_empty
@@ -230,38 +61,6 @@ def generate_max_properties_validator(maxProperties, **kwargs):
     return functools.partial(validate_max_properties, maximum=maxProperties)
 
 
-@skip_if_empty
-@skip_if_not_of_type(STRING)
-def validate_pattern(value, regex):
-    if not regex.match(value):
-        raise ValidationError(
-            "{0} did not match the pattern `{1}`.".format(value, regex.pattern),
-        )
-
-
-def generate_pattern_validator(pattern, **kwargs):
-    return functools.partial(validate_pattern, regex=re.compile(pattern))
-
-
-def generate_format_validator(format, **kwargs):
-    if format in registry:
-        return registry[format]
-    else:
-        raise ValueError('Unknown format {0}'.format(format))
-
-
-def validate_required(value):
-    if value is EMPTY:
-        raise ValidationError("This field is required.")
-
-
-def generate_required_validator(required, **kwargs):
-    if required:
-        return validate_required
-    else:
-        return noop
-
-
 validator_mapping = {
     'type': generate_type_validator,
     'multipleOf': generate_multiple_of_validator,
@@ -271,7 +70,7 @@ validator_mapping = {
     'maxLength': generate_max_length_validator,
     'minItems': generate_min_items_validator,
     'maxItems': generate_max_items_validator,
-    'uniqueItems': generate_unique_items_generator,
+    'uniqueItems': generate_unique_items_validator,
     'enum': generate_enum_validator,
     'minProperties': generate_min_properties_validator,
     'maxProperties': generate_max_properties_validator,
@@ -286,24 +85,17 @@ def validate_schema(obj, validators, inner=None):
     Given a json-like object to validate, and a dictionary of validators, apply
     the validators to the object.
     """
-    errors = collections.defaultdict(list)
+    with ErrorCollection(inner=inner) as errors:
+        if '$ref' in validators:
+            ref_ = validators.pop('$ref', {})
+            for k, v in ref_.validators.items():
+                validators.setdefault(k, v)
 
-    if '$ref' in validators:
-        ref_ = validators.pop('$ref', {})
-        for k, v in ref_.validators.items():
-            validators.setdefault(k, v)
-
-    for key, validator in validators.items():
-        try:
-            validator(obj)
-        except ValidationError as err:
-            errors[key].extend(list(err.messages))
-
-    if errors:
-        if inner:
-            raise SafeNestedValidationError(dict(errors))
-        else:
-            raise ValueError('Invalid:\n' + prettify_errors(errors))
+        for key, validator in validators.items():
+            try:
+                validator(obj)
+            except ValidationError as err:
+                errors[key].extend(list(err.messages))
 
 
 def validate_properties(obj, key, validators):
@@ -372,6 +164,21 @@ def construct_schema_validators(schema, context):
     """
     Given a schema object, construct a dictionary of validators needed to
     validate a response matching the given schema.
+
+    Special Cases:
+        - $ref:
+            These validators need to be Lazily evaluating so that circular
+            validation dependencies do not result in an infinitely deep
+            validation chain.
+        - properties:
+            These validators are meant to apply to properties of the object
+            being validated rather than the object itself.  In this case, we
+            need recurse back into this function to generate a dictionary of
+            validators for the property.
+        - items:
+            Validating items implies that we are working with an iterable of
+            things to be validated and that we may have to validate each
+            individual array item in some different way.
     """
     validators = {}
     if '$ref' in schema:
@@ -380,6 +187,7 @@ def construct_schema_validators(schema, context):
             context,
         )
     if 'properties' in schema:
+        # I don't know why this set intersection is enforced...?  Why did I do this.
         intersection = set(schema['properties'].keys()).intersection(schema.keys())
         assert not intersection
 
